@@ -1,7 +1,7 @@
 import unittest
 import os
 from pathlib import Path
-from itertools import groupby, product, islice
+from itertools import product, islice
 from tablemap import Conn
 
 
@@ -56,51 +56,6 @@ class TestGeneral(unittest.TestCase):
 
         self.assertEqual(conn['t1_sum'].list(), conn['t1_sum2'].list())
 
-    def test_full_join(self):
-        def add_col3(rs1, rs2):
-            # inner join
-            if rs1 and rs2:
-                for r1 in rs1:
-                    for r2 in rs2:
-                        r1['col3'] = r2['col3']
-                        yield r1
-            # left join
-            elif rs1 and not rs2:
-                for r1 in rs1:
-                    r1['col3'] = None
-                    yield r1
-            # right join
-            elif not rs1 and rs2:
-                for r2 in rs2:
-                    r2['col2'] = None
-                    yield r2
-
-        conn = Conn(self.dbfile)
-        conn['t1'] = table1
-        conn['t2'] = table2
-
-        conn['t1_col3'] = conn['t1'].by('col1')\
-            .merge(add_col3, conn['t2'].by('col1'))
-
-        self.assertEqual(conn['t1_col3'].list(), [
-            {'col1': 'a', 'col2': 4, 'col3': None},
-            {'col1': 'a', 'col2': 5, 'col3': None},
-            {'col1': 'b', 'col2': 1, 'col3': 1},
-            {'col1': 'c', 'col2': 3, 'col3': None},
-            {'col1': 'c', 'col2': 4, 'col3': None},
-            {'col1': 'c', 'col2': 7, 'col3': None},
-            {'col1': 'd', 'col2': 2, 'col3': 3},
-            {'col1': 'd', 'col2': 2, 'col3': 2},
-            {'col1': 'e', 'col2': None, 'col3': 3},
-            {'col1': 'e', 'col2': None, 'col3': 4}]
-        )
-
-        table2_1 = sorted(table2, key=lambda r: r['col1'])
-        conn['t1_col3_1'] = conn['t1'].by('col1')\
-            .merge(add_col3, groupby(table2_1, lambda r: r['col1']))
-
-        self.assertEqual(conn['t1_col3'].list(), conn['t1_col3_1'].list())
-
     def test_concat(self):
         conn = Conn(self.dbfile)
         conn['t1'] = table1
@@ -112,6 +67,14 @@ class TestGeneral(unittest.TestCase):
         self.assertEqual(conn['t1'].list() + conn['t1'].list(),
                          conn['t1_double'].list())
 
+    def test_df(self):
+        import pandas as pd
+        conn = Conn(self.dbfile)
+        conn['t1'] = table1
+        df = pd.DataFrame(conn['t1'].iter())
+        conn['t1_copy'] = df.to_dict('records')
+        self.assertEqual(conn['t1'].list(), conn['t1_copy'].list())
+
 
 class TestBy(unittest.TestCase):
     def setUp(self):
@@ -214,8 +177,6 @@ class TestDerivs(unittest.TestCase):
 
         self.assertEqual(len(conn['t1_2'].list()), 5)
 
-    
-
     def test_update(self):
         conn = Conn(self.dbfile)
         conn['t1'] = table1
@@ -294,21 +255,25 @@ class TestDerivs(unittest.TestCase):
         conn['t1_2'] = conn['t1'].distinct('col1', 'col2')
         self.assertEqual(len(conn['t1_2'].list()), 7)
 
-    def test_flatten(self):
-        conn = Conn(self.dbfile)
-        conn['t2'] = table2
-        self.assertEqual(conn['t2'].by('col1').flatten().list(), [
-            {'col1': 'b', 'col3': 1},
-            {'col1': 'd', 'col3': 3},
-            {'col1': 'd', 'col3': 2},
-            {'col1': 'e', 'col3': 3},
-            {'col1': 'e', 'col3': 4}]
-        )
 
-        self.assertEqual(conn['t2'].by('col1', 'col3').flatten().list(), [
-            {'col1': 'b', 'col3': 1},
-            {'col1': 'd', 'col3': 2},
-            {'col1': 'd', 'col3': 3},
-            {'col1': 'e', 'col3': 3},
-            {'col1': 'e', 'col3': 4}]
+class TestOrder(unittest.TestCase):
+    def setUp(self):
+        self.dbfile = 'test/sample.db'
+
+    def tearDown(self):
+        if Path(self.dbfile).is_file():
+            os.remove(self.dbfile)
+
+    def test_order(self):
+        conn = Conn(self.dbfile)
+        conn['t1'] = table1
+
+        [g1, g2, g3, g4] = [list(xs) for _, xs in conn['t1'].order(
+            'col1', 'col2 desc').group('col1').iter()]
+        self.assertEqual(
+            g1, [{'col1': 'a', 'col2': 5}, {'col1': 'a', 'col2': 4}]
         )
+        self.assertEqual(g2, [{'col1': 'b', 'col2': 1}])
+        self.assertEqual(g3, [{'col1': 'c', 'col2': 7}, {
+                         'col1': 'c', 'col2': 4}, {'col1': 'c', 'col2': 3}])
+        self.assertEqual(g4, [{'col1': 'd', 'col2': 2}])
